==> 0.1.0.tar.gz: lib/scroll_wheel/ScrollWheelTimeSelector.dart <==
import 'package:dotup_flutter_widgets/animations/BlinkingWidget.dart';
import 'package:dotup_flutter_widgets/utils/utils.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import 'ScrollWheelTimeSelectorController.dart';

class ScrollWheelTimeSelector extends StatefulWidget {
  // with ColorChangerMixin implements IColor {
  final ValueSetter<TimeOfDay> onTimeChanged;
  late final List<String> _hours;
  late final List<String> _minutes;
  late final FixedExtentScrollController _hourController;
  late final FixedExtentScrollController _minuteController;
  late final bool _infiniteScroll;
  late final double _height;
  late final Duration? notificationDelay;
  final ScrollWheelTimeSelectorController controller;

  ScrollWheelTimeSelector({
    Key? key,
    required this.onTimeChanged,
    List<String>? hours,
    List<String>? minutes,
    TimeOfDay? time,
    bool infiniteScroll = true,
    double height = 60,
    Duration? notificationDelay,
    required this.controller,
    // bool withNull = false,
  }) : super(key: key) {
    // hours
    _hours = hours == null ? List.generate(24, (index) => index.toString()) : hours;
    if (time == null) {
      _hours.insert(0, '--');
    }

    // minutes
    _minutes = minutes == null ? List.from(['00', '15', '30', '45']) : minutes;

    if (time == null) {
      _minutes.insert(0, '--');
    }

    final initialHour = time == null ? 0 : _hours.indexOf(time.hour.toString());
    final initialMinute = time == null ? 0 : _minutes.indexOf(time.minute.toString().padLeft(2, '0'));

    _hourController = FixedExtentScrollController(initialItem: initialHour);
    _minuteController = FixedExtentScrollController(initialItem: initialMinute);
    _infiniteScroll = infiniteScroll;
    _height = height;
    this.notificationDelay = notificationDelay ?? Duration(seconds: 2);
  }

  @override
  _ScrollWheelTimeSelectorState createState() => _ScrollWheelTimeSelectorState();
}

class _ScrollWheelTimeSelectorState extends State<ScrollWheelTimeSelector> {
  late final DelayedValue<TimeOfDay?> _delayedValue;
  // final ValueSetter<TimeOfDay> onTimeChanged;
  // late final List<String> _hours;
  // late final List<String> _minutes;
  // late final FixedExtentScrollController _hourController;
  // late final FixedExtentScrollController _minuteController;
  // late final bool _infiniteScroll;
  // late final double _height;
  // late final DelayedValue<TimeOfDay?> _delayedValue;
  // final ScrollWheelTimeSelectorController controller;

  @override
  void initState() {
    super.initState();

    _delayedValue = DelayedValue<TimeOfDay?>(
      delay: widget.notificationDelay ?? Duration(milliseconds: 400),
      callback: _onTimeChanged,
    );
  }

  @override
  dispose() {
    widget._hourController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final defaultTextStyle = DefaultTextStyle.of(context).style;
    // final textStyle = defaultTextStyle.
    return ChangeNotifierProvider.value(
      value: widget.controller,
      builder: (context, child) {
        if (widget._infiniteScroll) {
          return _getInfiniteWheels(defaultTextStyle);
        } else {
          return _getWheels(defaultTextStyle);
        }
      },
    );

    // final _style = Theme.of(context).textTheme.headline6;
    // return Center(
    //   child: ConstrainedBox(
    //     constraints: BoxConstraints(
    //       // Set height to one line, otherwise the whole vertical space is occupied.
    //       maxHeight: _style!.fontSize! * 4,
    //     ),
    //     child: ListWheelScrollView.useDelegate(
    //       itemExtent: _style!.fontSize!,
    //       magnification: 1.2,
    //       useMagnifier: true,
    //       physics: FixedExtentScrollPhysics(),
    //       diameterRatio: 1.1,
    //       squeeze: 1,
    //       childDelegate: ListWheelChildLoopingListDelegate(
    //         children: List<Widget>.generate(
    //           24,
    //           (index) => Text('${index + 1}', style: _style),
    //         ),
    //       ),
    //     ),
    //   ),
    // );
  }

  Widget _getInfiniteWheels(TextStyle defaultTextStyle) {
    return Container(
      height: widget._height,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          // Container(
          //   width: 50,
          //   child: ListWheelScrollView.useDelegate(
          //     controller: _hourController,
          //     itemExtent: 20,
          //     childDelegate: ListWheelChildLoopingListDelegate(
          //       children: _hours
          //           .map((e) => Text(
          //                 e,
          //                 style: defaultTextStyle,
          //               ))
          //           .toList(),
          //     ),
          //     magnification: 1.5,
          //     useMagnifier: true,
          //     physics: FixedExtentScrollPhysics(),
          //     diameterRatio: 1.1,
          //     squeeze: 1,
          //     onSelectedItemChanged: (index) => _delayedValue.update(_getTime()),
          //   ),
          // ),
          _getScrollViewDelegateContainer(widget._hourController, widget._hours, defaultTextStyle),
          Padding(
            padding: const EdgeInsets.only(bottom: 8.0),
            child: Text(':'),
          ),
          _getScrollViewDelegateContainer(widget._minuteController, widget._minutes, defaultTextStyle),
          // Container(
          //   width: 50,
          //   child: ListWheelScrollView.useDelegate(
          //     controller: _minuteController,
          //     itemExtent: 20,
          //     childDelegate: ListWheelChildLoopingListDelegate(
          //       children: _minutes.map(
          //         (e) {
          //           return ChangeNotifierProvider.value(
          //             value: controller,
          //             builder: (context, child) {
          //               return BlinkingWidget(
          //                 duration: Duration(milliseconds: 500),
          //                 // disabled: true,
          //                 // repeats: 3,
          //                 endColor: Colors.red,
          //                 builder: (BuildContext context, Color color) {
          //                   return Text('Hey', style: defaultTextStyle.copyWith(color: color));
          //                 },
          //               );
          //               // return Text(
          //               //   e,
          //               //   style: defaultTextStyle,
          //               // );
          //             },
          //           );
          //         },
          //       ).toList(),
          //     ),
          //     magnification: 1.5,
          //     useMagnifier: true,
          //     physics: FixedExtentScrollPhysics(),
          //     diameterRatio: 1.1,
          //     squeeze: 1,
          //     onSelectedItemChanged: (index) => _delayedValue.update(_getTime()),
          //   ),
          // ),
        ],
      ),
    );
  }

  Widget _getWheels(TextStyle defaultTextStyle) {
    return Container(
      height: widget._height,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          // Container(
          //   width: 50,
          //   child: ListWheelScrollView(
          //     controller: _hourController,
          //     itemExtent: 20,
          //     children: _hours
          //         .map((e) => Text(
          //               e,
          //               style: defaultTextStyle,
          //             ))
          //         .toList(),
          //     magnification: 1.5,
          //     useMagnifier: true,
          //     physics: FixedExtentScrollPhysics(),
          //     diameterRatio: 1.1,
          //     squeeze: 1,
          //     onSelectedItemChanged: (index) => _delayedValue.update(_getTime()),
          //   ),
          // ),
          _getScrollViewContainer(widget._hourController, widget._hours, defaultTextStyle),
          Padding(
            padding: const EdgeInsets.only(bottom: 8.0),
            child: Text(':'),
          ),
          _getScrollViewContainer(widget._minuteController, widget._minutes, defaultTextStyle),
          // Container(
          //   width: 50,
          //   child: ListWheelScrollView(
          //     controller: _minuteController,
          //     itemExtent: 20,
          //     children: _minutes.map((e) {
          //       return ListenableProvider.value(
          //         value: controller,
          //         builder: (context, child) {
          //           return Text(
          //             e,
          //             style: defaultTextStyle,
          //           );
          //         },
          //       );

          //       // return Text(
          //       //   e,
          //       //   style: defaultTextStyle,
          //       // );
          //     }).toList(),
          //     magnification: 1.5,
          //     useMagnifier: true,
          //     physics: FixedExtentScrollPhysics(),
          //     diameterRatio: 1.1,
          //     squeeze: 1,
          //     onSelectedItemChanged: (index) => _delayedValue.update(_getTime()),
          //   ),
          // ),
        ],
      ),
    );
  }

  Container _getScrollViewContainer(
    FixedExtentScrollController scrollController,
    List<String> items,
    TextStyle textStyle,
  ) {
    return Container(
      width: 50,
      child: ListWheelScrollView(
        controller: scrollController,
        itemExtent: 20,
        children: items.map(
          (e) {
            return Consumer<ScrollWheelTimeSelectorController>(
              builder: (context, controller, child) {
                return BlinkingWidget(
                  duration: Duration(milliseconds: 500),
                  disabled: controller.disabled,
                  // repeats: 3,
                  beginColor: textStyle.color,
                  endColor: Colors.red,
                  //   beginColor: Colors.white,
                  builder: (BuildContext context, Color color) {
                    return Text(e, style: textStyle.copyWith(color: color));
                  },
                );
              },
            );
          },
        ).toList(),
        magnification: 1.5,
        useMagnifier: true,
        physics: FixedExtentScrollPhysics(),
        diameterRatio: 1.1,
        squeeze: 1,
        onSelectedItemChanged: (index) => _delayedValue.update(_getTime()),
      ),
    );
  }

  Container _getScrollViewDelegateContainer(
    FixedExtentScrollController scrollController,
    List<String> items,
    TextStyle textStyle,
  ) {
    return Container(
      width: 50,
      child: ListWheelScrollView.useDelegate(
        controller: scrollController,
        itemExtent: 20,
        childDelegate: ListWheelChildLoopingListDelegate(
          children: items.map(
            (e) {
              return Consumer<ScrollWheelTimeSelectorController>(
                builder: (context, controller, child) {
                  return BlinkingWidget(
                    duration: Duration(milliseconds: 500),
                    disabled: controller.disabled,
                    // repeats: 3,
                    beginColor: textStyle.color,
                    endColor: Colors.red,
                    //   beginColor: Colors.white,
                    builder: (BuildContext context, Color color) {
                      return Text(e, style: textStyle.copyWith(color: color));
                    },
                  );
                },
              );
            },
          ).toList(),
        ),
        magnification: 1.5,
        useMagnifier: true,
        physics: FixedExtentScrollPhysics(),
        diameterRatio: 1.1,
        squeeze: 1,
        onSelectedItemChanged: (index) => _delayedValue.update(_getTime()),
      ),
    );
    // return ListWheelScrollView.useDelegate(
    //   controller: scrollController,
    //   itemExtent: 20,
    //   childDelegate: ListWheelChildLoopingListDelegate(
    //     children: items.map(
    //       (e) {
    //         return ChangeNotifierProvider.value(
    //           value: controller,
    //           builder: (context, child) {
    //             return BlinkingWidget(
    //               duration: Duration(milliseconds: 500),
    //               // disabled: true,
    //               // repeats: 3,
    //               endColor: Colors.red,
    //               builder: (BuildContext context, Color color) {
    //                 return Text(e, style: textStyle.copyWith(color: color));
    //               },
    //             );
    //           },
    //         );
    //       },
    //     ).toList(),
    //   ),
    // );
  }

  TimeOfDay? _getTime() {
    final h = widget._hours[widget._hourController.selectedItem % widget._hours.length];
    final m = widget._minutes[
        widget._minuteController.selectedItem % widget._minutes.length]; // _minutes[_minuteController.selectedItem];

    if (h == '--' || m == '--') {
      return null;
    } else {
      return TimeOfDay(hour: int.parse(h), minute: int.parse(m));
    }
  }

  @override
  void changeColor(Color color) {
    // TODO: implement changeColor
  }

  void _onTimeChanged(TimeOfDay? value) {
    if (value == null) {
      return;
    }

    widget.onTimeChanged(value);
  }
}

==> 0.2.2.tar.gz: lib/scroll_wheel/ScrollWheelTimeSelector.dart <==
import 'package:dotup_flutter_widgets/animations/BlinkingWidget.dart';
import 'package:dotup_flutter_widgets/dotup_flutter_widgets.dart';
import 'package:dotup_flutter_widgets/utils/utils.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import 'ScrollWheelTimeSelectorController.dart';

class ScrollWheelTimeSelector extends StatefulWidget {
  // with ColorChangerMixin implements IColor {
  final ValueSetter<TimeOfDay> onTimeChanged;
  late final List<String> _hours;
  late final List<String> _minutes;
  late final FixedExtentScrollController _hourController;
  late final FixedExtentScrollController _minuteController;
  late final bool _infiniteScroll;
  late final double _height;
  late final Duration? notificationDelay;
  final Color? color;
  final ScrollWheelTimeSelectorController controller;

  ScrollWheelTimeSelector({
    Key? key,
    required this.onTimeChanged,
    List<String>? hours,
    List<String>? minutes,
    TimeOfDay? time,
    bool infiniteScroll = true,
    double height = 60,
    Duration? notificationDelay,
    required this.controller,
    this.color,
    // bool withNull = false,
  }) : super(key: key) {
    // hours
    _hours = hours == null ? List.generate(24, (index) => index.toString()) : hours;
    if (time == null) {
      _hours.insert(0, '--');
    }

    // minutes
    _minutes = minutes == null ? List.from(['00', '15', '30', '45']) : minutes;

    if (time == null) {
      _minutes.insert(0, '--');
    }

    final initialHour = time == null ? 0 : _hours.indexOf(time.hour.toString());
    final initialMinute = time == null ? 0 : _minutes.indexOf(time.minute.toString().padLeft(2, '0'));

    _hourController = FixedExtentScrollController(initialItem: initialHour);
    _minuteController = FixedExtentScrollController(initialItem: initialMinute);
    _infiniteScroll = infiniteScroll;
    _height = height;
    this.notificationDelay = notificationDelay ?? Duration(seconds: 2);
  }

  @override
  _ScrollWheelTimeSelectorState createState() => _ScrollWheelTimeSelectorState();
}

class _ScrollWheelTimeSelectorState extends State<ScrollWheelTimeSelector> {
  late final DelayedValue<TimeOfDay?> _delayedValue;

  @override
  void initState() {
    super.initState();

    _delayedValue = DelayedValue<TimeOfDay?>(
      delay: widget.notificationDelay ?? Duration(milliseconds: 400),
      callback: _onTimeChanged,
    );
  }

  @override
  dispose() {
    widget._hourController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final TextStyle defaultTextStyle;

    if (widget.color == null) {
      defaultTextStyle = DefaultTextStyle.of(context).style;
    } else {
      defaultTextStyle = DefaultTextStyle.of(context).style.copyWith(color: widget.color);
    }

    // final textStyle = defaultTextStyle.
    return ChangeNotifierProvider.value(
      value: widget.controller,
      builder: (context, child) {
        if (widget._infiniteScroll) {
          return _getInfiniteWheels(defaultTextStyle);
        } else {
          return _getWheels(defaultTextStyle);
        }
      },
    );
  }

  Widget _getInfiniteWheels(TextStyle defaultTextStyle) {
    return Container(
      height: widget._height,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          _getScrollViewDelegateContainer(widget._hourController, widget._hours, defaultTextStyle),
          Padding(
            padding: const EdgeInsets.only(bottom: 8.0),
            child: Text(':'),
          ),
          _getScrollViewDelegateContainer(widget._minuteController, widget._minutes, defaultTextStyle),
        ],
      ),
    );
  }

  Widget _getWheels(TextStyle defaultTextStyle) {
    return Container(
      height: widget._height,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          _getScrollViewContainer(widget._hourController, widget._hours, defaultTextStyle),
          Padding(
            padding: const EdgeInsets.only(bottom: 8.0),
            child: Text(':'),
          ),
          _getScrollViewContainer(widget._minuteController, widget._minutes, defaultTextStyle),
        ],
      ),
    );
  }

  Container _getScrollViewContainer(
    FixedExtentScrollController scrollController,
    List<String> items,
    TextStyle textStyle,
  ) {
    return Container(
      width: 50,
      child: ListWheelScrollView(
        controller: scrollController,
        itemExtent: 20,
        children: items.map(
          (e) {
            return Consumer<ScrollWheelTimeSelectorController>(
              builder: (context, controller, child) {
                return BlinkingWidget(
                  duration: Duration(milliseconds: 500),
                  disabled: !controller.hasError,
                  // repeats: 3,
                  beginColor: textStyle.color,
                  endColor: Colors.red,
                  //   beginColor: Colors.white,
                  builder: (BuildContext context, Color color) {
                    return Text(e, style: textStyle.copyWith(color: color));
                  },
                );
              },
            );
          },
        ).toList(),
        magnification: 1.5,
        useMagnifier: true,
        physics: FixedExtentScrollPhysics(),
        diameterRatio: 1.1,
        squeeze: 1,
        onSelectedItemChanged: (index) => _delayedValue.update(_getTime()),
      ),
    );
  }

  Container _getScrollViewDelegateContainer(
    FixedExtentScrollController scrollController,
    List<String> items,
    TextStyle textStyle,
  ) {
    return Container(
      width: 50,
      child: ListWheelScrollView.useDelegate(
        controller: scrollController,
        itemExtent: 20,
        childDelegate: ListWheelChildLoopingListDelegate(
          children: items.map(
            (e) {
              return Consumer<ScrollWheelTimeSelectorController>(
                builder: (context, controller, child) {
                  return BlinkingWidget(
                    duration: Duration(milliseconds: 500),
                    disabled: !controller.hasError,
                    // repeats: 3,
                    beginColor: textStyle.color,
                    endColor: Colors.red,
                    //   beginColor: Colors.white,
                    builder: (BuildContext context, Color color) {
                      return Text(e, style: textStyle.copyWith(color: color));
                    },
                  );
                },
              );
            },
          ).toList(),
        ),
        magnification: 1.5,
        useMagnifier: true,
        physics: FixedExtentScrollPhysics(),
        diameterRatio: 1.1,
        squeeze: 1,
        onSelectedItemChanged: (index) => _delayedValue.update(_getTime()),
      ),
    );
  }

  TimeOfDay? _getTime() {
    final h = widget._hours[widget._hourController.selectedItem % widget._hours.length];
    final m = widget._minutes[widget._minuteController.selectedItem % widget._minutes.length];

    if (h == '--' || m == '--') {
      return null;
    } else {
      return TimeOfDay(hour: int.parse(h), minute: int.parse(m));
    }
  }

  void _onTimeChanged(TimeOfDay? value) {
    if (value == null) {
      return;
    }

    widget.onTimeChanged(value);
  }
}
